unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, wcrypt2;

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    FileMemo: TMemo;
    FileItem: TMenuItem;
    OpenItem: TMenuItem;
    EncryptItem: TMenuItem;
    DecryptItem: TMenuItem;
    SaveItem: TMenuItem;
    ExitItem: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    CloseItem: TMenuItem;
    ServiceItem: TMenuItem;
    InfoItem: TMenuItem;
    procedure OpenItemClick(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure EncryptItemClick(Sender: TObject);
    procedure SaveItemClick(Sender: TObject);
    procedure CloseItemClick(Sender: TObject);
    procedure GetPassword;
    procedure EncryptData;
    procedure DecryptData;
    procedure DecryptItemClick(Sender: TObject);
    procedure InfoItemClick(Sender: TObject);
  private
    { Private declarations }
    plaintext, ciphertext: string;
  public
    { Public declarations }
    hProv: HCRYPTPROV;
    key: HCRYPTKEY;
    encrypt: boolean;
  end;

var
  MainForm: TMainForm;

implementation

uses Passwd;

{$R *.DFM}

type algInfo = record
     algID: ALG_ID;
     dwBits: DWORD;
     dwNameLen: DWORD;
     szName: array[0..100] of char;
     end;

procedure TMainForm.OpenItemClick(Sender: TObject);
begin
OpenDialog1.Title := ' ';
OpenDialog1.Filter := 'Text files (*.txt)|*.TXT';
if OpenDialog1.Execute then
   begin
   plaintext := OpenDialog1.FileName;
   FileMemo.Lines.LoadFromFile(OpenDialog1.FileName);
   EncryptItem.Enabled := true;
   CloseItem.Enabled := true;
   end;
end;

procedure TMainForm.ExitItemClick(Sender: TObject);
begin
Close;
end;

procedure TMainForm.GetPassword;
var err: string;
begin
if not CryptAcquireContext(@hProv, nil, nil, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT) then
   begin
   case int64(GetLastError) of
   ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
   ERROR_NOT_ENOUGH_MEMORY: err := 'ERROR_NOT_ENOUGH_MEMORY';
   NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
   NTE_BAD_KEYSET: err := 'NTE_BAD_KEYSET';
   NTE_BAD_KEYSET_PARAM: err := 'NTE_BAD_KEYSET_PARAM';
   NTE_BAD_PROV_TYPE: err := 'NTE_BAD_PROV_TYPE';
   NTE_BAD_SIGNATURE: err := 'NTE_BAD_SIGNATURE';
   NTE_EXISTS: err := 'NTE_EXISTS';
   NTE_KEYSET_ENTRY_BAD: err := 'NTE_KEYSET_ENTRY_BAD';
   NTE_KEYSET_NOT_DEF: err := 'NTE_KEYSET_NOT_DEF';
   NTE_NO_MEMORY: err := 'NTE_NO_MEMORY';
   NTE_PROV_DLL_NOT_FOUND: err := 'NTE_PROV_DLL_NOT_FOUND';
   NTE_PROV_TYPE_ENTRY_BAD: err := 'NTE_PROV_TYPE_ENTRY_BAD';
   NTE_PROV_TYPE_NO_MATCH: err := 'NTE_PROV_TYPE_NO_MATCH';
   NTE_PROV_TYPE_NOT_DEF: err := 'NTE_PROV_TYPE_NOT_DEF';
   NTE_PROVIDER_DLL_FAIL: err := 'NTE_PROVIDER_DLL_FAIL';
   NTE_SIGNATURE_FILE_BAD: err := 'NTE_SIGNATURE_FILE_BAD';
   else err := 'Unknown error';
   end;
   MessageDlg('Error of CryptAcquireContext: '+err,
       mtError, [mbOK], 0);
   exit;
   end;
PasswordForm.Show;
end;

procedure TMainForm.EncryptItemClick(Sender: TObject);
begin
encrypt := true;
GetPassword;
end;

procedure TMainForm.EncryptData;
var i: integer;
    l: DWORD;
    err: string;
    data: PByte;
    inFile, outFile: file;
begin
SaveDialog1.Title := '   ?';
SaveDialog1.Filter := '  (*.*)|*.*|  (*.enc)|*.enc';
SaveDialog1.DefaultExt := 'enc';
if SaveDialog1.Execute then
   begin
   ciphertext := SaveDialog1.FileName;
   AssignFile(inFile, plaintext);
   AssignFile(outFile, ciphertext);
   reset(inFile, 1);
   rewrite(outFile, 1);
   GetMem(data, 512);
   while not eof(inFile) do
         begin
         BlockRead(inFile, data^, 512, l);
         if not CryptEncrypt(key, 0, eof(inFile), 0,
            data, @l, l) then
            begin
            case int64(GetLastError) of
            ERROR_INVALID_HANDLE: err := 'ERROR_INVALID_HANDLE';
            ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
            NTE_BAD_ALGID: err := 'NTE_BAD_ALGID';
            NTE_BAD_DATA: err := 'NTE_BAD_DATA';
            NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
            NTE_BAD_HASH: err := 'NTE_BAD_HASH';
            NTE_BAD_HASH_STATE: err := 'NTE_BAD_HASH_STATE';
            NTE_BAD_KEY: err := 'NTE_BAD_KEY';
            NTE_BAD_LEN: err := 'NTE_BAD_LEN';
            NTE_BAD_UID: err := 'NTE_BAD_UID';
            NTE_DOUBLE_ENCRYPT: err := 'NTE_DOUBLE_ENCRYPT';
            NTE_FAIL: err := 'NTE_FAIL';
            NTE_NO_MEMORY: err := 'NTE_NO_MEMORY';
            else err := 'Unknown error';
            end;
            MessageDlg('Error of CryptEncrypt: '+err,
                         mtError, [mbOK], 0);
            exit;
            end;
         BlockWrite(outFile, data^, l);
         end;
   FreeMem(data, 512);
   CloseFile(inFile);
   CloseFile(outFile);
   FileMemo.Lines.Clear;
   plaintext := '';
   ciphertext := '';
   EncryptItem.Enabled := false;
   end;
if not CryptReleaseContext(hProv, 0) then
   begin
   case int64(GetLastError) of
   ERROR_BUSY: err := 'ERROR_BUSY';
   ERROR_INVALID_HANDLE: err := 'ERROR_INVALID_HANDLE';
   ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
   NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
   NTE_BAD_UID: err := 'NTE_BAD_UID';
   else err := 'Unknown error';
   end;
   MessageDlg('Error of CryptReleaseContext: '+err,
       mtError, [mbOK], 0);
   end;
end;

procedure TMainForm.DecryptData;
var i: integer;
    l: DWORD;
    err: string;
    data: PByte;
    inFile, outFile: file;
begin
SaveDialog1.Title := '   ?';
SaveDialog1.Filter := '  (*.*)|*.*|  (*.dec)|*.dec|  (*.txt)|*.txt';
SaveDialog1.DefaultExt := 'dec';
if SaveDialog1.Execute then
   begin
   plaintext := SaveDialog1.FileName;
   AssignFile(inFile, ciphertext);
   AssignFile(outFile, plaintext);
   reset(inFile, 1);
   rewrite(outFile, 1);
   GetMem(data, 512);
   while not eof(inFile) do
         begin
         BlockRead(inFile, data^, 512, l);
         if not CryptDecrypt(key, 0, eof(inFile), 0,
            data, @l) then
            begin
            case int64(GetLastError) of
            ERROR_INVALID_HANDLE: err := 'ERROR_INVALID_HANDLE';
            ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
            NTE_BAD_ALGID: err := 'NTE_BAD_ALGID';
            NTE_BAD_DATA: err := 'NTE_BAD_DATA';
            NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
            NTE_BAD_HASH: err := 'NTE_BAD_HASH';
            NTE_BAD_HASH_STATE: err := 'NTE_BAD_HASH_STATE';
            NTE_BAD_KEY: err := 'NTE_BAD_KEY';
            NTE_BAD_LEN: err := 'NTE_BAD_LEN';
            NTE_BAD_UID: err := 'NTE_BAD_UID';
            NTE_DOUBLE_ENCRYPT: err := 'NTE_DOUBLE_ENCRYPT';
            NTE_FAIL: err := 'NTE_FAIL';
            NTE_NO_MEMORY: err := 'NTE_NO_MEMORY';
            else err := 'Unknown error';
            end;
            MessageDlg('Error of CryptDecrypt: '+err,
                         mtError, [mbOK], 0);
            exit;
            end;
         BlockWrite(outFile, data^, l);
         end;
   FreeMem(data, 512);
   CloseFile(inFile);
   CloseFile(outFile);
   FileMemo.Lines.LoadFromFile(plaintext);
   end;
end;

procedure TMainForm.SaveItemClick(Sender: TObject);
begin
SaveDialog1.Title := '  ';
SaveDialog1.Filter := '  (*.*)|*.*|  (*.txt)|*.txt';
SaveDialog1.DefaultExt := 'txt';
if SaveDialog1.Execute then
   begin
   plaintext := SaveDialog1.FileName;
   FileMemo.Lines.SaveToFile(plaintext);
   EncryptItem.Enabled := true;
   end;
end;

procedure TMainForm.CloseItemClick(Sender: TObject);
begin
FileMemo.Lines.Clear;
EncryptItem.Enabled := false;
plaintext := '';
ciphertext := '';
end;

procedure TMainForm.DecryptItemClick(Sender: TObject);
begin
OpenDialog1.Title := '   ';
OpenDialog1.Filter := 'Encrypted files (*.enc)|*.ENC|All files (*.*)|*.*';
if OpenDialog1.Execute then
   begin
   ciphertext := OpenDialog1.FileName;
   encrypt := false;
   GetPassword;
   end;
end;

function ProvTypeToStr(provType: DWORD): string;
begin
case provType of
PROV_RSA_FULL: ProvTypeToStr := 'RSA full provider';
PROV_RSA_SIG: ProvTypeToStr := 'RSA signature provider';
PROV_DSS: ProvTypeToStr := 'DSS provider';
PROV_DSS_DH: ProvTypeToStr := 'DSS and Diffie-Hellman provider';
PROV_FORTEZZA: ProvTypeToStr := 'Fortezza provider';
PROV_MS_EXCHANGE: ProvTypeToStr := 'MS Exchange provider';
PROV_RSA_SCHANNEL: ProvTypeToStr := 'RSA secure channel provider';
PROV_SSL: ProvTypeToStr := 'SSL provider';
else ProvTypeToStr := 'Unknown provider';
end;
end;

function ImpTypeToStr(it: DWORD): string;
begin
case it of
CRYPT_IMPL_HARDWARE: ImpTypeToStr := '';
CRYPT_IMPL_SOFTWARE: ImpTypeToStr := '';
CRYPT_IMPL_MIXED: ImpTypeToStr := '';
CRYPT_IMPL_UNKNOWN: ImpTypeToStr := '';
else ImpTypeToStr := ' ';
end;
end;

procedure TMainForm.InfoItemClick(Sender: TObject);
var i: DWORD;
    dwProvType, cbName, DataLen: DWORD;
    provName: array[0..200] of char;
    vers: array[0..3] of byte;
    impType: DWORD;
    ai: algInfo;
    err: string;
begin
i:= 0;
FileMemo.Clear;
while (CryptEnumProviders(i, nil, 0,
      @dwProvType, nil, @cbName)) do
      begin
      if CryptEnumProviders(i, nil, 0,
         @dwProvType, @provName, @cbName) then
         begin
         FileMemo.Lines.Add(': '+provName);
         FileMemo.Lines.Add(': '+IntToStr(dwProvType)+' - '+
                               ProvTypeToStr(dwProvType));
         if not CryptAcquireContext(@hProv,nil,provName,dwProvType,CRYPT_VERIFYCONTEXT)
         then
             begin
             case int64(GetLastError) of
             ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
             ERROR_NOT_ENOUGH_MEMORY: err := 'ERROR_NOT_ENOUGH_MEMORY';
             NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
             NTE_BAD_KEYSET: err := 'NTE_BAD_KEYSET';
             NTE_BAD_KEYSET_PARAM: err := 'NTE_BAD_KEYSET_PARAM';
             NTE_BAD_PROV_TYPE: err := 'NTE_BAD_PROV_TYPE';
             NTE_BAD_SIGNATURE: err := 'NTE_BAD_SIGNATURE';
             NTE_EXISTS: err := 'NTE_EXISTS';
             NTE_KEYSET_ENTRY_BAD: err := 'NTE_KEYSET_ENTRY_BAD';
             NTE_KEYSET_NOT_DEF: err := 'NTE_KEYSET_NOT_DEF';
             NTE_NO_MEMORY: err := 'NTE_NO_MEMORY';
             NTE_PROV_DLL_NOT_FOUND: err := 'NTE_PROV_DLL_NOT_FOUND';
             NTE_PROV_TYPE_ENTRY_BAD: err := 'NTE_PROV_TYPE_ENTRY_BAD';
             NTE_PROV_TYPE_NO_MATCH: err := 'NTE_PROV_TYPE_NO_MATCH';
             NTE_PROV_TYPE_NOT_DEF: err := 'NTE_PROV_TYPE_NOT_DEF';
             NTE_PROVIDER_DLL_FAIL: err := 'NTE_PROVIDER_DLL_FAIL';
             NTE_SIGNATURE_FILE_BAD: err := 'NTE_SIGNATURE_FILE_BAD';
             else err := 'Unknown error';
             end;
             MessageDlg('Error of CryptAcquireContext: '+err,
                               mtError, [mbOK], 0);
             exit;
             end;
         DataLen := 4;
         if not CryptGetProvParam(hProv, PP_VERSION, (@vers), @DataLen, 0)
         then
             begin
             case int64(GetLastError) of
             ERROR_INVALID_HANDLE: err := 'ERROR_INVALID_HANDLE';
             ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
             ERROR_MORE_DATA: err := 'ERROR_MORE_DATA';
             ERROR_NO_MORE_ITEMS: err := 'ERROR_NO_MORE_ITEMS';
             NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
             NTE_BAD_TYPE: err := 'NTE_BAD_TYPE';
             NTE_BAD_UID: err := 'NTE_BAD_UID';
             else err := 'Unknown error';
             end;
             MessageDlg('Error of CryptGetProvParam: '+err,mtError,[mbOK],0);
             exit
             end;
         FileMemo.Lines.Add(': '+chr(vers[1]+$30)+'.'+chr(vers[0]+$30));
         if not CryptGetProvParam(hProv, PP_IMPTYPE, (@impType), @DataLen, 0)
         then
             begin
             case int64(GetLastError) of
             ERROR_INVALID_HANDLE: err := 'ERROR_INVALID_HANDLE';
             ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
             ERROR_MORE_DATA: err := 'ERROR_MORE_DATA';
             ERROR_NO_MORE_ITEMS: err := 'ERROR_NO_MORE_ITEMS';
             NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
             NTE_BAD_TYPE: err := 'NTE_BAD_TYPE';
             NTE_BAD_UID: err := 'NTE_BAD_UID';
             else err := 'Unknown error';
             end;
             MessageDlg('Error of CryptGetProvParam: '+err,mtError,[mbOK],0);
             exit
             end;
         FileMemo.Lines.Add(' : '+ImpTypeToStr(impType));
         FileMemo.Lines.Add(' :');
         DataLen := sizeof(ai);
         if not CryptGetProvParam(hProv, PP_ENUMALGS, (@ai), @DataLen, CRYPT_FIRST)
         then
             begin
             case int64(GetLastError) of
             ERROR_INVALID_HANDLE: err := 'ERROR_INVALID_HANDLE';
             ERROR_INVALID_PARAMETER: err := 'ERROR_INVALID_PARAMETER';
             ERROR_MORE_DATA: err := 'ERROR_MORE_DATA';
             ERROR_NO_MORE_ITEMS: err := 'ERROR_NO_MORE_ITEMS';
             NTE_BAD_FLAGS: err := 'NTE_BAD_FLAGS';
             NTE_BAD_TYPE: err := 'NTE_BAD_TYPE';
             NTE_BAD_UID: err := 'NTE_BAD_UID';
             else err := 'Unknown error';
             end;
             MessageDlg('Error of CryptGetProvParam: '+err,mtError,[mbOK],0);
             exit
             end;
         with ai do
         FileMemo.Lines.Add(szName+#9+'  - '+IntToStr(dwBits)+' '+#9+
                                   'ID: '+IntToStr(AlgID));
         DataLen := sizeof(ai);
         while CryptGetProvParam(hProv, PP_ENUMALGS, (@ai), @DataLen, 0) do
               begin
               with ai do FileMemo.Lines.Add(szName+#9+'  - '
                    +IntToStr(dwBits)+' '+#9+'ID: '+IntToStr(AlgID));
               DataLen := sizeof(ai);
               end;
         FileMemo.Lines.Add('');
         CryptReleaseContext(hProv, 0);
         end;
      inc(i);
      end;
end;

end.
